unit chrWin;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Menus, TextEditor, Clipbrd, UnicodeData;

type
  TchrFrm = class(TForm)
    eChr: TEdit;
    eChrName: TLabeledEdit;
    eChrBlock: TLabeledEdit;
    eCode: TLabeledEdit;
    btnFindChr: TButton;
    pmFindChr: TPopupMenu;
    GotoCodepoint1: TMenuItem;
    Searchforstring1: TMenuItem;
    Browsecategory1: TMenuItem;
    pmEdits: TPopupMenu;
    Copytext1: TMenuItem;
    InsertText1: TMenuItem;
    Font1: TMenuItem;

    procedure FormResize(Sender: TObject);
    procedure UpdateSplitButton(Sender: TObject);
    procedure Copytext1Click(Sender: TObject);
    procedure InsertText1Click(Sender: TObject);
    procedure eChrEnter(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure pmEditsPopup(Sender: TObject);
    procedure Font1Click(Sender: TObject);
    procedure btnFindChrClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    { Private declarations }
    FEdit: TCustomEdit;
  public
    { Public declarations }
  end;

var
  chrFrm: TchrFrm;

implementation

uses mainWin, MultiInput, chrSearchResultWin;

{$R *.dfm}

function CodepointVerifier(const S: string): boolean;
var
  cp: integer;
begin
  result := (((Copy(S, 1, 2) = 'U+') and TryStrToInt('$' + Copy(S, 3), cp)) or
    TryStrToInt('$' + S, cp)) and (cp <= $FFFF);
end;

procedure TchrFrm.btnFindChrClick(Sender: TObject);

  function SameStr(const A, B: string): boolean;
  begin
    result := SysUtils.SameStr(StringReplace(A, '&', '', [rfReplaceAll]),
      StringReplace(B, '&', '', [rfReplaceAll]));
  end;

var
  v: string;
  matches: TIntegerArray;
  i: integer;
begin
  if SameStr(btnFindChr.Caption, pmFindChr.Items[0].Caption) then
  begin
    if length(eChr.Text) = 1 then
      v := UCD.GetChrCodepointStr(eChr.Text[1])
    else
      v := 'U+0000';
    if TMultiInputBox.TextInputBoxEx(Self, Caption, 'Please enter the codepoint to go to:', v, ecUpperCase, CodepointVerifier) then
    begin
      if UpperCase(Copy(v, 1, 2)) = 'U+' then
        Delete(v, 1, 2);
      mainFrm.CharacterInfo(Char(StrToInt('$' + v)));
    end;
  end
  else if SameStr(btnFindChr.Caption, pmFindChr.Items[1].Caption) then
  begin
    v := '';
    if TMultiInputBox.TextInputBox(Self, Caption, 'Please enter the string to search for:', v, ecUpperCase, false, [aoCapitalAZ, aoSpace]) then
    begin
      matches := UCD.SearchChrNames(v);
      if Length(matches) = 0 then
        MessageBox(Handle, 'No matching characters found.', PChar(Caption), MB_ICONINFORMATION or MB_OK)
      else
        with TchrSearchResultFrm.Create(nil) do
          try
            cbBlocks.Items.Clear;
            cbBlocks.Items.Add('Search result');
            cbBlocks.ItemIndex := 0;
            cbBlocks.OnChange := nil;
            lbChars.Items.BeginUpdate;
            lbChars.Items.Clear;
            for i := 0 to high(matches) do
              if matches[i] <= $FFFF then
                lbChars.Items.Add(Char(matches[i]));
            lbChars.Items.EndUpdate;
            ShowModal;
          finally
            Free;
          end;
    end;
  end
  else if SameStr(btnFindChr.Caption, pmFindChr.Items[2].Caption) then
  begin
    with TchrSearchResultFrm.Create(nil) do
      try
        cbBlocks.ItemIndex := 0;
        cbBlocksChange(Self);
        ShowModal;
      finally
        Free;
      end;
  end
end;

procedure TchrFrm.Copytext1Click(Sender: TObject);
begin
  Clipboard.AsText := FEdit.Text;
end;

procedure TchrFrm.eChrEnter(Sender: TObject);
begin
  if (Sender is TEdit) or (Sender is TLabeledEdit) then
    FEdit := Sender as TCustomEdit;
end;

procedure TchrFrm.Font1Click(Sender: TObject);
begin
  with TFontDialog.Create(self) do
    try
      Options := [fdNoSizeSel, fdNoStyleSel];
      Font.Name := eChr.Font.Name;
      if Execute then
        eChr.Font.Name := Font.Name;
      FormResize(Sender);
    finally
      Free;
    end;
end;

procedure TchrFrm.FormCreate(Sender: TObject);
begin
  FEdit := eChr;
end;

procedure TchrFrm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    Close;
end;

procedure TchrFrm.FormResize(Sender: TObject);
begin
  eChr.Font.Height := eChr.ClientHeight;
end;

procedure TchrFrm.InsertText1Click(Sender: TObject);
begin
  mainFrm.FKeyPressed := true;
  mainFrm.Editor.SelText := FEdit.Text;
end;

procedure TchrFrm.pmEditsPopup(Sender: TObject);
begin
  Font1.Visible := FEdit = eChr;
end;

procedure TchrFrm.UpdateSplitButton(Sender: TObject);
begin
  if Sender is TMenuItem then
  begin
    with Sender as TMenuItem do
    begin
      btnFindChr.Caption := Caption;
      btnFindChr.Hint := Hint;
    end;
    btnFindChr.Click;
  end;
end;

end.
